#include <string.h>
#include "global_vars.h"
#include "serial.h"
#include "options.h"
#include "error_handlers.h"
#include "sensors.h"

#define MSG_TOGGLE   MSG_USER

#define NUM_OF_SENSORS        11
#define NUM_OF_RETRIES        3
#define SENSORS_TO_TIME_OUT   3 //number of sensors that need to time out before the warning will be issued

typedef struct
{
   void (*formula)(int raw_data, char *buf);
   char sensor_screen_buf[80]; // calculate actual characters needed
   char command[5];
   int priority; // 1 = Highest
   int i; // counter
} SENSOR;

// procedure declarations
static int sensor_proc(int msg, DIALOG *d, int c);
static int toggle_proc(int msg, DIALOG *d, int c);
static int status_proc(int msg, DIALOG *d, int c);

static void engine_rpm_formula(int data, char *buf);
static void engine_load_formula(int data, char *buf);
static void coolant_temp_formula(int data, char *buf);
static void fuel_system_status_formula(int data, char *buf);
static void vehicle_speed_formula(int data, char *buf);
static void short_term_fuel_trim_formula(int data, char *buf);
static void intake_pressure_formula(int data, char *buf);
static void timing_advance_formula(int data, char *buf);
static void intake_air_temp_formula(int data, char *buf);
static void air_flow_rate_formula(int data, char *buf);
static void throttle_position_formula(int data, char *buf);

static int device_connected = FALSE;

static DIALOG sensor_dialog[] =
{
   /* (proc)            (x)  (y)  (w)  (h) (fg)     (bg)          (key) (flags)  (d1) (d2) (dp)                           (dp2) (dp3) */
   { d_clear_proc,      0,   0,   0,   0,  0,       C_WHITE,       0,    0,       0,   0,   NULL,                          NULL, NULL },
   { d_box_proc,        40,  13,  560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  17,  45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  0,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 20,  200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Engine RPM:",                 NULL, NULL },
   { sensor_proc,       328, 20,  260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, &(SENSOR){ engine_rpm_formula,           "N/A", "010C", 1, 0 } },
   { d_box_proc,        40,  49,  560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  53,  45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  1,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 56,  200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Calculated Load Value:",      NULL, NULL },
   { sensor_proc,       328, 56,  260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       1,   0,   NULL,                          NULL, &(SENSOR){ engine_load_formula,          "N/A", "0104", 1, 0 } },
   { d_box_proc,        40,  85,  560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  89,  45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  2,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 92,  200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Coolant Temperature:",        NULL, NULL },
   { sensor_proc,       328, 92,  260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       2,   0,   NULL,                          NULL, &(SENSOR){ coolant_temp_formula,         "N/A", "0105", 1, 0 } },
   { d_box_proc,        40,  121, 560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  125, 45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  3,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 128, 200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Fuel System Status:",         NULL, NULL },
   { sensor_proc,       328, 128, 260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       3,   0,   NULL,                          NULL, &(SENSOR){ fuel_system_status_formula,   "N/A", "0103", 1, 0 } },
   { d_box_proc,        40,  157, 560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  161, 45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  4,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 164, 200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Vehicle Speed:",              NULL, NULL },
   { sensor_proc,       328, 164, 260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       4,   0,   NULL,                          NULL, &(SENSOR){ vehicle_speed_formula,        "N/A", "010D", 1, 0 } },
   { d_box_proc,        40,  193, 560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  197, 45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  5,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 200, 200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Short Term Fuel Trim:",       NULL, NULL },
   { sensor_proc,       328, 200, 260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       5,   0,   NULL,                          NULL, &(SENSOR){ short_term_fuel_trim_formula, "N/A", "0106", 1, 0 } },
   { d_box_proc,        40,  229, 560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  233, 45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  6,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 236, 200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Intake Manifold Pressure:",   NULL, NULL },
   { sensor_proc,       328, 236, 260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       6,   0,   NULL,                          NULL, &(SENSOR){ intake_pressure_formula,      "N/A", "010B", 1, 0 } },
   { d_box_proc,        40,  265, 560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  269, 45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  7,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 272, 200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Timing Advance:",             NULL, NULL },
   { sensor_proc,       328, 272, 260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       7,   0,   NULL,                          NULL, &(SENSOR){ timing_advance_formula,       "N/A", "010E", 1, 0 } },
   { d_box_proc,        40,  301, 560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  305, 45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  8,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 308, 200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Intake Air Temp:",            NULL, NULL },
   { sensor_proc,       328, 308, 260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       8,   0,   NULL,                          NULL, &(SENSOR){ intake_air_temp_formula,      "N/A", "010F", 1, 0 } },
   { d_box_proc,        40,  337, 560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  341, 45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  9,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 344, 200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Air Flow Rate:",              NULL, NULL },
   { sensor_proc,       328, 344, 260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       9,   0,   NULL,                          NULL, &(SENSOR){ air_flow_rate_formula,        "N/A", "0110", 1, 0 } },
   { d_box_proc,        40,  373, 560, 32, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   NULL,                          NULL, NULL },
   { toggle_proc,       45,  377, 45,  24, C_BLACK, C_WHITE,       0,    D_EXIT,  10,  0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      120, 380, 200, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       0,   0,   "Absolute Throttle Position:", NULL, NULL },
   { sensor_proc,       328, 380, 260, 24, C_BLACK, C_LIGHT_GRAY,  0,    0,       10,  0,   NULL,                          NULL, &(SENSOR){ throttle_position_formula,    "N/A", "0111", 1, 0 } },
   { d_shadow_box_proc, 40,  419, 434, 32, C_BLACK, C_WHITE,       0,    0,       0,   0,   NULL,                          NULL, NULL },
   { d_rtext_proc,      48,  425, 88,  24, C_BLACK, C_WHITE,       0,    0,       0,   0,   "Port Status:",                NULL, NULL },
   { status_proc,       166, 425, 300, 24, C_BLACK, C_WHITE,       0,    0,       0,   0,   NULL,                          NULL, NULL },
   { d_button_proc,     490, 419, 110, 48, C_BLACK, C_DARK_YELLOW, 'm',  D_EXIT,  0,   0,   "&Main Menu",                  NULL, NULL },
   { NULL,              0,   0,   0,   0,  0,       0,             0,    0,       0,   0,   NULL,                          NULL, NULL }
};


int display_sensor_dialog()
{
   return do_dialog(sensor_dialog, -1);
}


int toggle_proc(int msg, DIALOG *d, int c)
{
   int ret;
   
   switch (msg)
   {
      case MSG_START:
         d->d2 = 1;
         if ((d->dp = calloc(4, sizeof(char))) == NULL)
         {
            sprintf(temp_error_buf, "Could not allocate enough memory for toggle button caption: %i", d->d1);
            fatal_error(temp_error_buf);
         }
         strcpy(d->dp, "ON");
         d->bg = C_GREEN;
         break;
   
      case MSG_TOGGLE:
         if (c == d->d1)
         {
            if (d->d2 == 0)
            {
               d->d2 = 1;
               strcpy(d->dp, "ON");
               d->bg = C_GREEN;
            }
            else
            {
               d->d2 = 0;
               strcpy(d->dp, "OFF");
               d->bg = C_DARK_YELLOW;
            }
      
            return D_REDRAWME;
         }
         break;
   
      case MSG_END:
         free(d->dp);
         d->dp = NULL;
         break;
   }
   
   ret = d_button_proc(msg, d, c);
   
   if (ret == D_CLOSE)
   {
      broadcast_dialog_message(MSG_TOGGLE, d->d1);
      return D_O_K;
   }
   
   return ret;
}


int status_proc(int msg, DIALOG *d, int c)
{
   switch (msg)
   {
      case MSG_START:
         if((d->dp = calloc(48, sizeof(char))) == NULL)
            fatal_error("Could not allocate enough memory for status control (sensors)");
         d->d1 = device_connected;
         d->d2 = -1;
         break;
   
      case MSG_END:
         free(d->dp);
         d->dp = NULL;
         break;
         
      case MSG_DRAW:
      {
         int circle_color;

         if ((comport.status == READY) && device_connected)
            circle_color = C_GREEN;
         else if ((comport.status == NOT_OPEN) || (comport.status == USER_IGNORED))
            circle_color = C_RED;
         else 
            circle_color = C_DARK_YELLOW;
         
         circlefill(screen, d->x-d->h/2, d->y+d->h/2-3, d->h/2-2, circle_color);
         circle(screen, d->x-d->h/2, d->y+d->h/2-3, d->h/2-2, C_BLACK);
         
         set_clip(screen, d->x, d->y, d->x+d->w-1, d->y+d->h-1);  // define screen area = dimensions of element
         clear_to_color(screen, d->bg); // erase the screen area
         set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1); // reset clipping
         break;
      }
      case MSG_IDLE:      
         if ((d->d1 != device_connected) || (d->d2 != comport.status))
         {
            d->d1 = device_connected;
            d->d2 = comport.status;
         
            if (comport.status == READY)
            {
               if (device_connected)
                  sprintf(d->dp, " COM%i is ready (device connected)", comport.number + 1);
               else
                  sprintf(d->dp, " COM%i is ready (device not responding)", comport.number + 1);
            }
            else
               sprintf(d->dp, " COM%i could not be opened", comport.number + 1);
         
            return D_REDRAWME;
         }
   }

   return d_text_proc(msg, d, c);
}


int sensor_proc(int msg, DIALOG *d, int c)
{
   static int current_sensor = 0; // sensor we're working on
   static int receiving_response = FALSE; // flag: receiving or sending
   static int num_of_sensors_timed_out = 0;
   static int ignore_device_not_connected = FALSE;
   static int retry_attempts = NUM_OF_RETRIES;
   static int num_of_disabled_sensors = 0;
   static char vehicle_response[512]; // try 33 (11 bytes * 2 + 10 spaces + CR + '/0')
   char comport_buffer[80]; // calculate actual value
   char temp_buf[64];
   int response_status = EMPTY; //status of the response: EMPTY, DATA, PROMPT
   int response_type;
   int ret = 0;
   SENSOR *sensor = (SENSOR *)d->dp3;

   switch (msg)
   {
      case MSG_START:
         d->dp = &sensor->sensor_screen_buf;
         sensor->i = 1;
         strcpy((sensor->sensor_screen_buf), "N/A");
         if (d->d1 == 0)
         {
            current_sensor = 0;
            num_of_disabled_sensors = 0;
            num_of_sensors_timed_out = 0;
            device_connected = FALSE;
            ignore_device_not_connected = FALSE;
            receiving_response = FALSE;
         }
         d->flags &= ~D_DISABLED;
         sprintf(temp_buf, "sensor%i", d->d1);
         push_config_state();
         set_config_file(options_file_name);
         if (get_config_int("sensors", temp_buf, 1) == 0)
            broadcast_dialog_message(MSG_TOGGLE, d->d1);
         pop_config_state();
         break;

      case MSG_END:
         sprintf(temp_buf, "sensor%i", d->d1);
         push_config_state();
         set_config_file(options_file_name);
         if (d->flags & D_DISABLED)
            set_config_int("sensors", temp_buf, 0);
         else
            set_config_int("sensors", temp_buf, 1);
         pop_config_state();
         if (d->d1 == NUM_OF_SENSORS - 1)
            stop_serial_timer();
         break;
   
      case MSG_TOGGLE:
         if(d->d1 == c)
         {
            if (d->flags & D_DISABLED)
            {
               d->flags &= ~D_DISABLED;
               num_of_disabled_sensors--;
               strcpy(sensor->sensor_screen_buf, "N/A");
            }
            else
            {
               d->flags |= D_DISABLED;
               num_of_disabled_sensors++;
               if (num_of_disabled_sensors == NUM_OF_SENSORS) // if all of the sensors are disabled
                  num_of_sensors_timed_out = 0; // reset timeout counter
            }

            return D_REDRAWME;
         }
         break;

      case MSG_DRAW:
         set_clip(screen, d->x, d->y, d->x+d->w-1, d->y+d->h-1);  // define screen area = dimensions of element
         clear_to_color(screen, d->bg); // erase the screen area
         set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1); // reset clipping
         break;

      case MSG_IDLE:
         if (d->d1 == current_sensor)
         {
            if (comport.status == READY)
            {
               if (!receiving_response)
               {
                  if (d->flags & D_DISABLED) // if the sensor is disabled
                  {
                     current_sensor = (current_sensor == NUM_OF_SENSORS - 1) ? 0 : current_sensor + 1; // go to the next sensor
                     return D_O_K;
                  }
                  sensor->i--;  // decrement the counter in dp1
                  if (sensor->i == 0)  // if it's our turn,
                  {
                     send_command(sensor->command); // send command for that particular sensor
                     receiving_response = TRUE; // now we're waiting for response
                     start_serial_timer(OBD_REQUEST_TIMEOUT); // start the timer
                     sensor->i = sensor->priority; // reset priority counter
                     vehicle_response[0] = '\0'; // clear string
                  }
                  else
                     /* if current_sensor is the last sensor, set it to 0; otherwise current_sensor++ */
                     current_sensor = (current_sensor == NUM_OF_SENSORS - 1) ? 0 : current_sensor + 1;
               }
               else  // if we are receiving response
               {
                  response_status = read_comport(comport_buffer);  // read comport
                  
                  if(response_status == DATA) // if data detected in com port buffer
                  {
                     strcat(vehicle_response, comport_buffer); // append contents of comport_buffer to vehicle_response
                  }
                  else if(response_status == PROMPT) // if '>' detected
                  {
                     device_connected = TRUE;
                     num_of_sensors_timed_out = 0;
                     receiving_response = FALSE; // we're not waiting for response any more
                     stop_serial_timer();
                     
                     strcat(vehicle_response, comport_buffer); // append contents of comport_buffer to vehicle_response
                     response_type = process_response(sensor->command, vehicle_response, MULTIPLE_LINE);

                     if (response_type == HEX_DATA)// HEX_DATA received
                     {
                        char delimiter_str[2];
                        sprintf(delimiter_str, "%c", SPECIAL_DELIMITER);
                        strtok(vehicle_response, delimiter_str);
                        sensor->formula(ascii_hex_to_int(vehicle_response, 4), vehicle_response); //plug the value into formula
                        strcpy(sensor->sensor_screen_buf, vehicle_response);  // and copy result in screen buffer
                        /* if current_sensor is the last sensor, set it to 0; otherwise current_sensor++ */
                        current_sensor = (current_sensor == NUM_OF_SENSORS - 1) ? 0 : current_sensor + 1;
                        retry_attempts = NUM_OF_RETRIES;
                     }
                     else
                     {
                        strcpy(sensor->sensor_screen_buf, "N/A");
                  
                        if(response_type == ERR_NO_DATA) // if we received "NO DATA", "N/A" wiil be printed
                        {
                           current_sensor = (current_sensor == NUM_OF_SENSORS - 1) ? 0 : current_sensor + 1; // next time poll next sensor
                           retry_attempts = NUM_OF_RETRIES;
                        }
                        
                        else if(response_type == BUS_ERROR) // if we received "BUS ERROR"
                        {
                           alert("Bus Error: OBDII bus is shorted to Vbatt or Ground.", NULL, NULL, "OK", NULL, 0, 0);
                           retry_attempts = NUM_OF_RETRIES;
                        }
                           
                        // if we received "BUS BUSY", "DATA ERROR", "<DATA ERROR", SERIAL_ERROR, or RUBBISH,
                        // try to re-send the request, do nothing if successful and alert user if failed:
                        else
                        {
                           if(retry_attempts > 0)
                           {
                              retry_attempts--;
                              return D_O_K;
                           }
                           else
                           {
                              switch (response_type)
                              {
                                 case BUS_BUSY:
                                    alert("OBD Bus Busy: could not read sensor", NULL, NULL, "OK", NULL, 0, 0);
                                    break;
                                 case DATA_ERROR: case DATA_ERROR2:
                                    alert("Data Error: there has been a loss of data.", "You may have a bad connection to the vehicle,", "check the cable.", "OK", NULL, 0, 0);
                                    break;
                                 case SERIAL_ERROR: case RUBBISH:
                                    alert("Serial Link Error: please check connection", "between computer and OBD interface.", NULL, "OK", NULL, 0, 0);
                                    break;
                              }
                              retry_attempts = NUM_OF_RETRIES; // reset the number of retry attempts
                           }
                        } // end of BUS_BUSY, DATA_ERROR, DATA_ERROR2, ERIAL_ERROR, and RUBBISH
                     }
                     return D_REDRAWME; //tell the parent control to redraw itself
                  }
               }
            }
            
            if (serial_time_out) // if timeout occured,
            {
               receiving_response = FALSE; // we're not waiting for a response any more
               strcpy(sensor->sensor_screen_buf, "N/A");
            
               if (num_of_sensors_timed_out >= SENSORS_TO_TIME_OUT)
               {
                  num_of_sensors_timed_out = 0;
                  device_connected = FALSE;
                  if  (!ignore_device_not_connected)
                  {
                     ret = alert3("Device is not responding.", "Please check that it is connected", "and the port settings are correct", "&OK", "&Configure Port", "&Ignore", 'o', 'c', 'i');
                     if (ret == 2)
                        display_options();   // let the user choose correct settings
                     else if (ret == 3)
                        ignore_device_not_connected = TRUE;
                  }
               }
               else
                  num_of_sensors_timed_out++;

               while (comport.status == NOT_OPEN)
               {
                  if (alert("Port is not ready.", "Please check that you specified the correct port", "and that no other application is using it", "&Configure Port", "&Ignore", 'c', 'i') == 1)
                     display_options(); // let the user choose correct settings
                  else
                     comport.status = USER_IGNORED;
               }
            
               stop_serial_timer();

               /* if current_sensor is the last sensor, set it to 0; otherwise current_sensor++ */
               current_sensor = (current_sensor == NUM_OF_SENSORS - 1) ? 0 : current_sensor + 1;
            
               return D_REDRAWME;
            }
         }
         break;
   } // end of switch (msg)

   if (d->flags & D_DISABLED)
   {
      strcpy(sensor->sensor_screen_buf, "not monitoring");
   }

   return d_text_proc(msg, d, c);  // pass all the messages on to d_text_proc, return
}


void engine_rpm_formula(int data, char *buf)
{
   if (system_of_measurements == METRIC)
      sprintf(buf, "%i r/min", (int)((float)data/4)); // perform necessary calculations, and print to buf
   else   // if the system is IMPERIAL
      sprintf(buf, "%i rpm", (int)((float)data/4)); // perform necessary calculations, and print to buf
}


void engine_load_formula(int data, char *buf)
{
      sprintf(buf, "%.1f%%", (float)data*100/255); // perform necessary calculations, and print to buf
}


void coolant_temp_formula(int data, char *buf)
{
   if (system_of_measurements == METRIC)
      sprintf(buf, "%i%c C", data-40, 0xB0); // perform necessary calculations, and print to buf
   else   // if the system is IMPERIAL
      sprintf(buf, "%i%c F", (int)(((float)data - 40)*9/5 + 32), 0xB0); // perform necessary calculations, and print to buf
}


void fuel_system_status_formula(int data, char *buf)
{
   data = data & 0xFF00;

   if (data == 0x0100)
      sprintf(buf, "OPEN LOOP");
   else if (data == 0x0200)
      sprintf(buf, "CLOSED LOOP");
   else if (data == 0x0400)
      sprintf(buf, "OPEN LOOP, DRIVING");
   else if (data == 0x0800)
      sprintf(buf, "OPEN LOOP, SYSTEM FAULT");
   else if (data == 0x1000)
      sprintf(buf, "CLOSED LOOP, OXYGEN SENSOR FAULT");
   else
      sprintf(buf, "UNKNOWN: %i", data);
}


void vehicle_speed_formula(int data, char *buf)
{
   if (system_of_measurements == METRIC)
      sprintf(buf, "%i km/h", data); // perform necessary calculations, and print to buf
   else   // if the system is IMPERIAL
      sprintf(buf, "%i mph", (int)((float)data/1.609)); // perform necessary calculations, and print to buf
}


void short_term_fuel_trim_formula(int data, char *buf)
{
   sprintf(buf, "%i%%", (int)(((float)data - 128)*100/128)); // perform necessary calculations, and print to buf
}


void intake_pressure_formula(int data, char *buf)
{
   if (system_of_measurements == METRIC)
      sprintf(buf, "%i kPa", data);
   else
      sprintf(buf, "%.1f in.hg.", (float)data*0.2953);
}


void timing_advance_formula(int data, char *buf)
{
   sprintf(buf, "%.1f%c", ((float)data-128)/2, 0xB0);
}


void intake_air_temp_formula(int data, char *buf)
{
   if (system_of_measurements == METRIC)
      sprintf(buf, "%i%c C", data-40, 0xB0); // perform necessary calculations, and print to buf
   else   // if the system is IMPERIAL
      sprintf(buf, "%i%c F", (int)(((float)data - 40)*9/5 + 32), 0xB0); // perform necessary calculations, and print to buf
}


void air_flow_rate_formula(int data, char *buf)
{
   if (system_of_measurements == METRIC)
      sprintf(buf, "%.1f g/s", (float)data*0.01);
   else
      sprintf(buf, "%.1f lb/min", (float)data*0.132276);
}


void throttle_position_formula(int data, char *buf)
{
   sprintf(buf, "%.1f%%", (float)data*100/255);
}
