#include <string.h>
#include <math.h>
#include <ctype.h>
#include "global_vars.h"
#ifdef ALLEGRO_WINDOWS
   #include <winalleg.h>
#else
   #include <dzcomm.h>
#endif
#include "serial.h"

#ifdef ALLEGRO_WINDOWS
   static HANDLE com_port;
#else
   static comm_port *com_port;
#endif


//timer interrupt handler for sensor data
static void serial_time_out_handler()
{
   serial_time_out = TRUE;
}
END_OF_STATIC_FUNCTION(serial_time_out_handler)


void start_serial_timer(int delay)
{
    install_int_ex(serial_time_out_handler, MSEC_TO_TIMER(delay));  // install the timer
    serial_time_out = FALSE;
}


void stop_serial_timer()
{
   remove_int(serial_time_out_handler);
   serial_time_out = FALSE;
}


void serial_module_init()
{
   #ifndef ALLEGRO_WINDOWS
      dzcomm_init();
   #endif
   /* all variables and code used inside interrupt handlers must be locked */
   LOCK_VARIABLE(serial_time_out);
   LOCK_FUNCTION(serial_time_out_handler);
}


void serial_module_shutdown()
{
   close_comport();
   
   #ifndef ALLEGRO_WINDOWS
      //dzcomm_closedown(); //dzcomm_closedown() will be automatically called on exit
   #endif
}


int open_comport()
{
   #ifdef ALLEGRO_WINDOWS
      DCB dcb;
      COMMTIMEOUTS timeouts;
      char port_number_str[6];
   #endif
   
   if (comport.status == READY)    // if the comport is open,
      close_comport();    // close it

   #ifdef ALLEGRO_WINDOWS
      sprintf(port_number_str, "COM%i", comport.number + 1);
      com_port = CreateFile(port_number_str, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);
      if (com_port == INVALID_HANDLE_VALUE)
      {
         comport.status = NOT_OPEN; //port was not open
         return -1; // return error
      }
      GetCommState(com_port, &dcb);
      dcb.BaudRate = 9600;
      dcb.Parity = NOPARITY;
      dcb.StopBits = ONESTOPBIT;
      SetCommState(com_port, &dcb);
      
      timeouts.ReadIntervalTimeout = MAXWORD;
      timeouts.ReadTotalTimeoutMultiplier = 0;
      timeouts.ReadTotalTimeoutConstant = 0;
      timeouts.WriteTotalTimeoutMultiplier = 0;
      timeouts.WriteTotalTimeoutConstant = 0;
      SetCommTimeouts(com_port, &timeouts);
   #else
      com_port = comm_port_init(comport.number);
      comm_port_set_baud_rate(com_port, _9600);
      comm_port_set_parity(com_port, NO_PARITY);
      comm_port_set_data_bits(com_port, BITS_8);
      comm_port_set_stop_bits(com_port, STOP_1);
      comm_port_set_flow_control(com_port, NO_CONTROL);
      if (comm_port_install_handler(com_port) != 1)
      {
         comport.status = NOT_OPEN; //port was not open
         return -1; // return error
      }
   #endif
   serial_time_out = FALSE;
   comport.status = READY;
   
   return 0; // everything is okay
}


void close_comport()
{
   if (comport.status == READY)    // if the comport is open, close it
   {
      #ifdef ALLEGRO_WINDOWS
         PurgeComm(com_port, PURGE_TXCLEAR|PURGE_RXCLEAR);
         CloseHandle(com_port);
      #else
         comm_port_flush_output(com_port);
         comm_port_flush_input(com_port);
         comm_port_uninstall(com_port);
      #endif
   }
   comport.status = NOT_OPEN;
}


void send_command(char *command)
{
   #ifdef ALLEGRO_WINDOWS
      DWORD bytes_written;
      
      PurgeComm(com_port, PURGE_TXCLEAR|PURGE_RXCLEAR);
      WriteFile(com_port, command, strlen(command), &bytes_written, 0);
      WriteFile(com_port, "\r", 1, &bytes_written, 0);
   #else
      comm_port_flush_output(com_port);
      comm_port_flush_input(com_port);
      comm_port_command_send(com_port, command);
   #endif
}


int read_comport(char *response)
{
   char *prompt_pos = NULL;

   #ifdef ALLEGRO_WINDOWS
      DWORD bytes_read = 0;
      DWORD errors;
      COMSTAT stat;
      
      response[0] = '\0';
      ClearCommError(com_port, &errors, &stat);
      if (stat.cbInQue > 0)
         ReadFile(com_port, response, stat.cbInQue, &bytes_read, 0);
      response[bytes_read] = '\0';
   #else
      int i = 0;
      
      while((response[i] = comm_port_test(com_port)) != -1) // while the serial buffer is not empty, read comport
         i++;
      response[i] = '\0'; // terminate string, erase -1
   #endif
   
   prompt_pos = strchr(response, '>');
   if (prompt_pos != NULL)
   {
      *prompt_pos = '\0'; // erase ">"
      return PROMPT;      // command prompt detected
   }
   else if (strlen(response) == 0)  // if the string is empty,
      return EMPTY;
   else                         //otherwise,
      return DATA;
}


int process_response(const char *cmd_sent, char *msg_received, int parser_flag)
{
   int i = 0;
   int j = 0;
   int echo_on = TRUE; //echo status
   int is_hex_num = TRUE;
   char temp_buf[80];

   for(i = 0; i < strlen(cmd_sent); i++)
   {
      if (cmd_sent[i] != msg_received[i])    // if the characters are not the same,
      {
         echo_on = FALSE;  // say that echo is off
         break;            // break out of the loop
      }
   }

   if (echo_on == TRUE)  //if echo is on
   {
      send_command("ate0"); // turn off the echo
      start_serial_timer(AT_TIMEOUT);
      while ((read_comport(temp_buf) != PROMPT) && !serial_time_out);
      stop_serial_timer();
      i = strlen(cmd_sent) + 1; //initialize counter with the start of data (skip the request)
   }
   else //if echo is off
      i = 0; //initialize counter with start of recieved message buffer

   for(; i < strlen(msg_received)-2; i++) //loop to copy data
   {
      if (msg_received[i] > 32)  // if the character is not a special character
      {
         msg_received[i] = toupper(msg_received[i]);
         //if it's a valid hex digit, 0-9 or A-F
         if ((((msg_received[i] >= 48) && (msg_received[i] <= 57)) || ((msg_received[i] >= 65) && (msg_received[i] <= 70))) == FALSE)
         {
            if (msg_received[i] == '<')
               return DATA_ERROR2; // inform calling function we got "<DATA ERROR"
            else
               is_hex_num = FALSE;
         }

         msg_received[j] = msg_received[i]; // rewrite response
         j++;
      }
      else if (((msg_received[i] == 13) || (msg_received[i] == 10)) && (parser_flag == MULTIPLE_LINE)) // if the character is a CR and we are parsing a multiple line response
      {
         msg_received[j] = SPECIAL_DELIMITER; // replace CR with SPECIAL_DELIMITER
         j++;
      }
   }
   msg_received[j] = 0; // terminate the string

   if (is_hex_num)
      return HEX_DATA;

   if (strcmp(msg_received, "BUSBUSY") == 0)
      return BUS_BUSY;
   if (strcmp(msg_received, "BUSERROR") == 0)
      return BUS_ERROR;
   if (strcmp(msg_received, "DATAERROR") == 0)
      return DATA_ERROR;
   if ((strcmp(msg_received, "NODATA") == 0) || (strcmp(msg_received, "BUSINIT:...ERROR") == 0))
      return ERR_NO_DATA;
   if (strcmp(msg_received, "?") == 0)
      return SERIAL_ERROR;

   return RUBBISH;
}


int ascii_hex_to_int(const char *char_string, int start_pos)
{
   int i, j = 0;
   int ret = 0;

   for(i = strlen(char_string) - 1; i >= start_pos; i--)    //start from start_pos usually to skip first two bytes (four ascii chars) -- header
   {
      //if it's a valid hex digit, 0-9 or A-F
      if (((char_string[i] >= 48) && (char_string[i] <= 57)) || ((char_string[i] >= 65) && (char_string[i] <= 70)))
      {
         if (isdigit(char_string[i]))    // if it's a digit,
            ret = ret + (char_string[i] - 48) * pow(16, j);
         else   // if it's a character,
            ret = ret + (char_string[i] - 55) * pow(16, j);
         j++;
      }
    }

   return ret;  // return the value in decimal
}
